package com.wtkj.service.impl;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.wtkj.entity.*;
import com.wtkj.mapper.ProjectMapper;
import com.wtkj.service.*;
import lombok.AllArgsConstructor;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.system.entity.Dept;
import org.springblade.system.feign.ISysClient;
import org.springblade.system.user.entity.User;
import org.springblade.system.user.feign.IUserClient;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static com.wtkj.config.MagicValue.ZERO;

/**
 * @author Blizzard
 * @create at 2023-09-13 16:32
 * @describe
 */
@Service
@AllArgsConstructor
public class AsyncServiceImpl implements IAsyncService {

	private final ITypeAndStageService typeAndStageService;
	private final IProjectStageService stageService;
	private final IProjectAuthService projectAuthService;
	private final ISysClient sysClient;
	private final IUserClient userClient;
	private final ITaskLogService taskLogService;
	private final IIndexMessageService indexMessageService;
	private final IProjectGroupService projectGroupService;
	private final ProjectMapper projectMapper;

	@Override
	@Async("asyncPoolTaskExecutor")
	public void createProject(Project project, BladeUser user) {
		//1.项目类型  --> 使用类型对应的模版阶段创建项目阶段
		Long projectType = project.getProjectType();
		List<TemplateTypeAndStage> templateStage = typeAndStageService.getStages(projectType);
		//2.创建项目阶段
		if (!CollectionUtils.isEmpty(templateStage)) {
			List<ProjectStage> stagesList = new ArrayList<>();
			templateStage.forEach(template -> {
				//项目的阶段记录表
				Long id = IdWorker.getId();
				ProjectStage stage = new ProjectStage();
				stage.setId(id);
				stage.setProjectId(project.getId());
				stage.setName(template.getName());
				stage.setSort(template.getSort());
				stage.setCreateUser(project.getCreateUser());
				stage.setCreateDept(project.getCreateDept());
				stage.setCreateTime(DateUtil.now());
				stage.setUpdateTime(DateUtil.now());
				stage.setUpdateUser(project.getUpdateUser());
				//添加默认阶段
				if (template.getSort() == 1) {
					project.setCurrentStage(id);
					projectMapper.updateById(project);
				}
				stagesList.add(stage);
			});
			if (!CollectionUtils.isEmpty(stagesList)) {
				stageService.saveBatch(stagesList);
			}
		}

		//权限
		String deptIdStr = user.getDeptId();
		if (deptIdStr != null) {
			Long deptId = Func.firstLong(deptIdStr);
			ProjectAuth auth = new ProjectAuth();
			auth.setUserId(user.getUserId());
			R<Dept> rpc = sysClient.getDept(deptId);
			if (rpc.isSuccess()) {
				String ancestors = rpc.getData().getAncestors();
				if (ancestors != null && ancestors.contains(String.valueOf(ZERO))) {
					List<Long> parentIds = Func.toLongList(ancestors);
					if (!CollectionUtils.isEmpty(parentIds)) {
						if (parentIds.size() > 1) {
							auth.setTopDept(parentIds.get(1));
						} else {
							auth.setTopDept(Long.valueOf(deptIdStr));
						}
					}
				}
			}
			auth.setUserDept(deptId);
			auth.setProjectId(project.getId());
			projectAuthService.save(auth);
		}

	}

	@Async("asyncPoolTaskExecutor")
	@Override
	public void removeProject(List<Long> longList) {

	}

	@Override
	@Async("asyncPoolTaskExecutor")
	public void createTask(Task task) {
		Long projectId = task.getProjectId();
		Project project = projectMapper.selectById(projectId);
		if (project != null) {
			//1.任务日志
			Long createUser = task.getCreateUser();
			if (createUser != null) {
				R<User> userR = userClient.userInfoById(createUser);
				if (userR.isSuccess()) {
					List<TaskLog> logs = new ArrayList<>();

					TaskLog log = new TaskLog();
					log.setTaskId(task.getId());
					log.setContent(userR.getData().getName() + " 创建了任务");
					log.setRelatedUser(String.valueOf(userR.getData().getId()));
					log.setCreateDept(task.getCreateDept());
					log.setCreateUser(task.getCreateUser());
					log.setUpdateUser(task.getUpdateUser());
					log.setCreateTime(DateUtil.now());
					log.setUpdateTime(DateUtil.now());
					logs.add(log);
					Long orgDeptId = null;
					if (task.getOrgDeptId() != null && task.getOrgDeptId() > 0L) {
						orgDeptId = task.getOrgDeptId();
						R<Dept> dept = sysClient.getDept(orgDeptId);
						if (dept.isSuccess()) {
							TaskLog taskLog = new TaskLog();
							taskLog.setTaskId(task.getId());
							taskLog.setContent(userR.getData().getName() + " 指定了执行单位为" + dept.getData().getDeptName());
							taskLog.setRelatedUser(String.valueOf(userR.getData().getId()));
							taskLog.setCreateDept(task.getCreateDept());
							taskLog.setCreateUser(task.getCreateUser());
							taskLog.setUpdateUser(task.getUpdateUser());
							try {
								Thread.sleep(3000);
							} catch (InterruptedException e) {
								System.out.println("休眠异常:" + e.getMessage());
							}
							taskLog.setCreateTime(DateUtil.now());
							taskLog.setUpdateTime(DateUtil.now());
							logs.add(taskLog);

							ProjectAuth auth = new ProjectAuth();
							auth.setTopDept(orgDeptId);
							auth.setProjectId(task.getProjectId());
							auth.setCreateDept(task.getCreateDept());
							auth.setCreateUser(task.getCreateUser());
							auth.setUpdateUser(task.getUpdateUser());
							auth.setCreateTime(DateUtil.now());
							auth.setUpdateTime(DateUtil.now());
							projectAuthService.save(auth);
						}
					}
					taskLogService.saveBatch(logs);
				}
			}
			// todo 2.任务下发通知
		}
	}

	@Override
	@Async("asyncPoolTaskExecutor")
	public void updateTask(Integer type, Long userId, Task task) {
		R<User> userR = userClient.userInfoById(userId);
		if (userR.isSuccess()) {
			List<TaskLog> logs = new ArrayList<>();
			List<ProjectAuth> auths = new ArrayList<>();
			List<ProjectGroup> groups = new ArrayList<>();

			Long orgDeptId = task.getOrgDeptId();
			if (type == 1) {
				R<Dept> dept = sysClient.getDept(orgDeptId);
				if (dept.isSuccess()) {
					TaskLog log = new TaskLog();
					log.setTaskId(task.getId());
					log.setContent(userR.getData().getName() + " 指定了执行单位为" + dept.getData().getDeptName());
					log.setRelatedUser(String.valueOf(userR.getData().getId()));
					log.setCreateDept(task.getCreateDept());
					log.setCreateUser(task.getCreateUser());
					log.setUpdateUser(task.getUpdateUser());
					log.setCreateTime(DateUtil.now());
					log.setUpdateTime(DateUtil.now());
					logs.add(log);

					ProjectAuth auth = new ProjectAuth();
					auth.setTopDept(orgDeptId);
					auth.setProjectId(task.getProjectId());
					auths.add(auth);
				}
			} else if (type == 2) {
				Long executeDept = task.getExecuteDept();
				R<Dept> dept1 = sysClient.getDept(executeDept);
				if (dept1.isSuccess()) {
					TaskLog log1 = new TaskLog();
					log1.setTaskId(task.getId());
					log1.setContent(userR.getData().getName() + " 指定了任务执行部门为" + dept1.getData().getDeptName());
					log1.setRelatedUser(String.valueOf(userR.getData().getId()));
					log1.setCreateDept(task.getCreateDept());
					log1.setCreateUser(task.getCreateUser());
					log1.setUpdateUser(task.getUpdateUser());
					log1.setCreateTime(DateUtil.now());
					log1.setUpdateTime(DateUtil.now());
					logs.add(log1);

					ProjectAuth auth = new ProjectAuth();
					auth.setTopDept(orgDeptId);
					auth.setUserDept(executeDept);
					auth.setProjectId(task.getProjectId());
					auths.add(auth);
				}
			} else if (type == 3) {
				Long projectManager = task.getProjectManager();
				R<User> userR1 = userClient.userInfoById(projectManager);
				if (userR1.isSuccess()) {
					TaskLog log2 = new TaskLog();
					log2.setTaskId(task.getId());
					log2.setContent(userR.getData().getName() + " 指定了项目经理为 " + userR1.getData().getName());
					List<Long> users = new ArrayList<>();
					users.add(userR.getData().getId());
					users.add(userR1.getData().getId());
					log2.setRelatedUser(Func.join(users));
					log2.setCreateDept(task.getCreateDept());
					log2.setCreateUser(task.getCreateUser());
					log2.setUpdateUser(task.getUpdateUser());
					log2.setCreateTime(DateUtil.now());
					log2.setUpdateTime(DateUtil.now());
					logs.add(log2);

					String deptStr = userR1.getData().getDeptId();
					if (StringUtil.isNotBlank(deptStr) && orgDeptId != null) {
						//项目权限
						Long deptId = Func.firstLong(deptStr);
						ProjectAuth auth = new ProjectAuth();
						auth.setTopDept(orgDeptId);
						auth.setUserDept(deptId);
						auth.setProjectId(task.getProjectId());
						auth.setUserId(projectManager);
						auths.add(auth);

						//项目组
						ProjectGroup group = new ProjectGroup();
						group.setInviteDept(null);
						group.setProjectId(task.getProjectId());
						group.setStageId(task.getStageId());
						group.setBeInvitedDept(orgDeptId);
						group.setUserId(projectManager);
						groups.add(group);
					}
				}
			} else if (type == 4) {
				List<Long> userIds = Func.toLongList(task.getExecuteUser());
				List<String> str = new ArrayList<>();
				userIds.forEach(id -> {
					R<User> rpc = userClient.userInfoById(id);
					if (rpc.isSuccess()) {
						str.add(rpc.getData().getName());

						String deptStr = rpc.getData().getDeptId();
						if (StringUtil.isNotBlank(deptStr) && orgDeptId != null) {
							Long deptId = Func.firstLong(deptStr);
							ProjectAuth auth = new ProjectAuth();
							auth.setTopDept(orgDeptId);
							auth.setUserDept(deptId);
							auth.setProjectId(task.getProjectId());
							auth.setUserId(id);
							auths.add(auth);

							ProjectGroup group = new ProjectGroup();
							group.setInviteDept(null);
							group.setProjectId(task.getProjectId());
							group.setStageId(task.getStageId());
							group.setBeInvitedDept(orgDeptId);
							group.setUserId(id);
							groups.add(group);
						}
					}
				});
				if (!CollectionUtils.isEmpty(str)) {
					TaskLog log3 = new TaskLog();
					log3.setTaskId(task.getId());
					log3.setContent(userR.getData().getName() + " 指定了任务执行者 " + Func.join(str));
					List<Long> users = new ArrayList<>();
					users.add(userR.getData().getId());
					users.addAll(userIds);
					log3.setRelatedUser(Func.join(users));
					log3.setCreateDept(task.getCreateDept());
					log3.setCreateUser(task.getCreateUser());
					log3.setUpdateUser(task.getUpdateUser());
					log3.setCreateTime(DateUtil.now());
					log3.setUpdateTime(DateUtil.now());
					logs.add(log3);
				}
			} else if (type == 5) {
				Long first = task.getFirstCheckUser();
				Long second = task.getSecondCheckUser();
				List<Long> checkUsers = Arrays.asList(first, second);
				List<String> checks = new ArrayList<>();
				checkUsers.forEach(id -> {
					R<User> rpc = userClient.userInfoById(id);
					if (rpc.isSuccess()) {
						checks.add(rpc.getData().getName());

						String deptStr = rpc.getData().getDeptId();
						if (StringUtil.isNotBlank(deptStr) && orgDeptId != null) {
							Long deptId = Func.firstLong(deptStr);
							ProjectAuth auth = new ProjectAuth();
							auth.setTopDept(orgDeptId);
							auth.setUserDept(deptId);
							auth.setProjectId(task.getProjectId());
							auth.setUserId(id);
							auths.add(auth);

							ProjectGroup group = new ProjectGroup();
							group.setInviteDept(null);
							group.setProjectId(task.getProjectId());
							group.setStageId(task.getStageId());
							group.setBeInvitedDept(orgDeptId);
							group.setUserId(id);
							groups.add(group);
						}
					}
				});
				if (!CollectionUtils.isEmpty(checks)) {
					TaskLog log4 = new TaskLog();
					log4.setTaskId(task.getId());
					log4.setContent(userR.getData().getName() + " 指定了任务审查者 " + Func.join(checks));
					List<Long> users = new ArrayList<>();
					users.add(userR.getData().getId());
					users.addAll(checkUsers);
					log4.setRelatedUser(Func.join(users));

					log4.setCreateDept(task.getCreateDept());
					log4.setCreateUser(task.getCreateUser());
					log4.setUpdateUser(task.getUpdateUser());
					log4.setCreateTime(DateUtil.now());
					log4.setUpdateTime(DateUtil.now());
					logs.add(log4);
				}
			} else if (type == 6) {
				//todo  提交、确认任务
			} else if (type == 7) {

			}

			taskLogService.saveBatch(logs);
			if (!CollectionUtils.isEmpty(auths)) {
				projectAuthService.saveBatch(auths);
			}
			if (!CollectionUtils.isEmpty(groups)) {
				projectGroupService.saveBatch(groups);
			}
		}
	}

	@Override
	@Async("asyncPoolTaskExecutor")
	public void submitTaskMessage(Long userId, Task task) {
		Long firstCheckUser = task.getFirstCheckUser();
		Long secondCheckUser = task.getSecondCheckUser();

		Long projectId = task.getProjectId();
		String title = task.getTitle();
		R<User> userR = userClient.userInfoById(userId);
		if (userR.isSuccess()) {
			List<IndexMessage> list = new ArrayList<>();
			IndexMessage message = new IndexMessage();
			message.setCategory(1);
			message.setToUser(firstCheckUser);
			message.setContent(userR.getData().getName() + "上传了" + title + "任务的文件，请查阅");
			message.setOpenUrl("/task");
			message.setCreateDept(task.getCreateDept());
			message.setCreateUser(userId);
			message.setUpdateUser(userId);
			message.setCreateTime(DateUtil.now());
			message.setUpdateTime(DateUtil.now());
			list.add(message);

			IndexMessage msg = new IndexMessage();
			msg.setCategory(1);
			msg.setToUser(secondCheckUser);
			msg.setContent(userR.getData().getName() + "上传了" + title + "任务的文件，请查阅");
			msg.setOpenUrl("/task");
			msg.setCreateDept(task.getCreateDept());
			msg.setCreateUser(userId);
			msg.setUpdateUser(userId);
			msg.setCreateTime(DateUtil.now());
			msg.setUpdateTime(DateUtil.now());
			list.add(msg);

			indexMessageService.saveBatch(list);
		}
	}


}
